import requests
import os
import time
from bs4 import BeautifulSoup
import urllib3
from concurrent.futures import ThreadPoolExecutor
from alive_progress import alive_bar

urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)

def get_href_list(html_string):
    href_list = []
    soup = BeautifulSoup(html_string, 'html.parser')
    links = soup.find_all('a')
    for link in links:
        href = link.get('href')
        href_list.append(href)
    return href_list

def save_image_from_url(image_url, save_path):
    # Create directory if it doesn't exist
    os.makedirs(os.path.dirname(save_path), exist_ok=True)
    
    # Send a GET request to the image URL
    response = requests.get(image_url, verify=False)
    
    # Check if the request was successful (status code 200)
    if response.status_code == 200:
        # Open a new file in binary write mode
        with open(save_path, 'wb') as f:
            # Write the content of the response (image) to the file
            f.write(response.content)
    else:
        print(f"Failed to download image from {image_url}")

# Function to retrieve images for a specific camera ID and date range
def retrieve_images(base_url, camera_id, unix_timestamp, days_before=0, days_after=0):
    # Convert Unix timestamp to date
    date = time.strftime("%Y/%j", time.gmtime(unix_timestamp))
    
    # Construct URL for the specific camera ID
    camera_url = base_url + f"{camera_id}/{date}/"
    
    # Make request to check if camera directory exists
    response = requests.get(camera_url, verify=False)
    if response.status_code == 200:
        print(f"Camera ID {camera_id} found.")
        # Create directory to store images
        os.makedirs(camera_id, exist_ok=True)
        
        # Calculate start and end dates
        start_date = unix_timestamp - (days_before * 86400)  # 86400 seconds in a day        
        # Iterate over days and retrieve images
        for day_offset in range(-days_before, days_after + 1):
            current_date = start_date + (day_offset * 86400)
            current_date_str = time.strftime("%Y/%j", time.gmtime(current_date))
            day_url = base_url + f"{camera_id}/{current_date_str}/"
            
            # Retrieve images for the current day
            response = requests.get(day_url, verify=False)
            if response.status_code == 200:
                # Save images to local directory
                image_dir = f"{camera_id}/{current_date_str}/"
                os.makedirs(image_dir, exist_ok=True)
                download_images_from_directory(day_url, image_dir)
                print(f"Images for {current_date_str} retrieved.")
            else:
                print(f"No images found for {current_date_str}.")
    else:
        print(f"Camera ID {camera_id} not found.")

# Function to download images from a directory
def download_images_from_directory(directory_url, local_directory):
    skip = 3
    response = requests.get(directory_url, verify=False)
    if response.status_code == 200:
        hours = get_href_list(response.text)[4:]
        with alive_bar(int(24 * 300/skip)) as bar:
            # Basically sending multiple requests at one time. 
            with ThreadPoolExecutor() as executor:
                futures = []
                for hour in hours:
                    hour_url = directory_url + hour
                    hour_response = requests.get(hour_url, verify=False)
                    images = get_href_list(hour_response.text)[5:]
                    for image in images[::skip]:
                        image_url = hour_url + image
                        futures.append(executor.submit(save_image_from_url, image_url, f"./{local_directory}{hour}{image}"))
                for future in futures:
                    future.result()
                    bar()
    else:
        print("???")

# The variables defined are all the videos to be scraped from ALERTCalifornia. 

frames_5 = "https://proxy.acequia.io/proxy?url=https://map.alertwildfire.ucsd.edu/fireframes5/frames/"
redis_5 = "https://proxy.acequia.io/proxy?url=https://map.alertwildfire.ucsd.edu/fireframes5/digitalpath-redis/"
scraped_items = [["Axis-PilotHillEldorado2", 1662531625, 1, 1, frames_5], ["Axis-RockCreek", 1662531625, 1, 1, frames_5], 
                 ["Axis-MeadowBrook1", 1662531625, 1, 1, frames_5], ["Axis-BaldMtnButte1", 1662531625, 1, 1, frames_5],
                 ["Axis-HotchkissHill", 1662531625, 1, 1, frames_5]]
# # CACCCC_wishon
scraped_items += [["Axis-Idyllwild1", 1662562970, 2, 4, frames_5], ["Axis-RangerPeak2", 1662562970, 2, 4, frames_5]]
# # scraped_items += [["Axis-AntelopeYreka1", 1659931235, 2, 2, frames_5], ["Axis-FortJones1", 1659931235, 2, 2, frames_5], ["Axis-HerdPeak2", 1659931235, 2, 2, frames_5]]
# #kelly
scraped_items += [["Axis-WildFrontier1", 1692321607, 4, 4, redis_5], ["Axis-HighDivide1", 1692321607, 4, 4, redis_5]]
# # blue Creek
scraped_items += [["Axis-DeerHorn1", 1694311934, 1, 5, redis_5], ["Axis-RedDelNorte1", 1694311934, 1, 5, redis_5]]
#
 
futures = []

# Executes the code and runs all the concurrent requests at once
for item in scraped_items:
    [camera_id, unix_timestamp, days_before, days_after, base_url] = item
    with ThreadPoolExecutor() as executor_main:
        futures.append(executor_main.submit(retrieve_images(base_url, camera_id, unix_timestamp, days_before, days_after)))
for future in futures:
    future.result()

