import os
import cv2
from datetime import datetime, timedelta

# The function of this script wasto turn the scraped images into a more readable format; aka a video. 

def create_video(startdate, enddate, path, output_path='output.mp4', fps=25):
    # Convert start and end dates to datetime objects
    start_year, start_day, start_hour = startdate
    end_year, end_day, end_hour = enddate
    start_date = datetime.strptime(f"{start_year}-{start_day}-{start_hour}", "%Y-%j-%H")
    end_date = datetime.strptime(f"{end_year}-{end_day}-{end_hour}", "%Y-%j-%H")
    
    # Initialize video writer with codec for MP4
    fourcc = cv2.VideoWriter_fourcc(*'mp4v')
    out = cv2.VideoWriter(output_path, fourcc, fps, (1920, 1080))
    
    # Loop through each year directory
    for year_folder in os.listdir(path):
        year_path = os.path.join(path, year_folder)
        if os.path.isdir(year_path):
            year = int(year_folder)
            # Check if the year is within the specified range
            if start_date.year <= year <= end_date.year:
                # Loop through each day directory
                for day_folder in os.listdir(year_path):
                    day_path = os.path.join(year_path, day_folder)
                    if os.path.isdir(day_path):
                        day = int(day_folder)
                        # Check if the day is within the specified range and year
                        current_date = datetime(year, 1, 1) + timedelta(days=day - 1)
                        if start_date.date() <= current_date.date() <= end_date.date():
                            # Loop through hour directories within the day directory
                            for hour_folder in os.listdir(day_path):
                                hour_path = os.path.join(day_path, hour_folder)
                                if os.path.isdir(hour_path):
                                    hour = int(hour_folder)
                                    # Check if the hour is within the specified range and day
                                    current_hour = current_date + timedelta(hours=hour)
                                    if start_date <= current_hour <= end_date:
                                        # Loop through images in the hour directory and add to video
                                        for image_file in os.listdir(hour_path):
                                            image_path = os.path.join(hour_path, image_file)
                                            img = cv2.imread(image_path)
                                            if img is not None:
                                                out.write(img)
    
    # Release video writer
    out.release()

# Function to play the video
def play_video(video_path):
    cap = cv2.VideoCapture(video_path)
    while True:
        ret, frame = cap.read()
        if not ret:
            break
        cv2.imshow('Video Playback', frame)
        if cv2.waitKey(25) & 0xFF == ord('q'):
            break
    
    # Release capture and destroy any open windows
    cap.release()
    cv2.destroyAllWindows()

# Example usage:
startdate = (2022, 250, "20")  # Year, Day
enddate = (2022, 250, "23")    # Year, Day
paths = ['Axis-Wildfires/Axis-MosquitoFire/Axis-RockCreek']
output_path = 'output.mp4'

for path in paths:
    print("Creating video for " + path)
    output = path + ".mp4"
    create_video(startdate, enddate, path, output)
