import cv2
import random
import json
from pointcloud import *

current_frames = [[], []]
current_ret = [0,0]
selected_points = [None, None]
current_frame = 0
previous_frame = 0

# used for data visualization
def generate_random_color():
    r = random.randint(0, 255)
    g = random.randint(0, 255)
    b = random.randint(0, 255)
    return (r, g, b)

def generate_random_colors(num_colors):
    colors = []
    for _ in range(num_colors):
        colors.append(generate_random_color())
    return colors

colors = generate_random_colors(255)

# used to register clicks for manual optical flow
def mouse_callback(event, x, y, flags, param):
    global selected_points
    if event == cv2.EVENT_LBUTTONDOWN:
        if x < param['cap1_width']:  # Clicked on the first video frame
            # param['points']['video1'].append((x, y))
            selected_points[0] = (x, y)
        else:  # Clicked on the second video frame
            # param['points']['video2'].append((x - param['cap1_width'], y))
            selected_points[1] = ((x - param['cap1_width']), y)

        size_0 = (640, 360)
        param['cf'] = current_frames
        param["cr"] = current_ret
        redraw(param)

# update CV2 imagery function
def redraw(params):
    global selected_points
    global current_frame
    global previous_frame
    global colors

    [ret1, frame1, ret2, frame2] = [params["cr"][0], params["cf"][0], params["cr"][1], params["cf"][1]]

    if not ret1 or not ret2:
        return

    # Resize frames to have the same height
    height = max(frame1.shape[0], frame2.shape[0])
    frame1 = cv2.resize(frame1, (int(frame1.shape[1] * height / frame1.shape[0]), height))
    frame2 = cv2.resize(frame2, (int(frame2.shape[1] * height / frame2.shape[0]), height))
    print(frame1.shape)
    # Display frames side by side
    frame = cv2.hconcat([frame1, frame2])

    # Draw points currently selected
    if selected_points[0] != None:
        if current_frame in params['points']['video1'][0].keys():
            cv2.circle(frame, selected_points[0], 2, colors[len(params['points']['video1'][0][current_frame])], -1)
    if selected_points[1] != None:
        if current_frame in params['points']['video1'][0].keys():
            cv2.circle(frame, (selected_points[1][0]+frame1.shape[1], selected_points[1][1]), 2, colors[len(params['points']['video1'][0][current_frame])], -1)

    # Draw clicked points on the frames
    if current_frame in params['points']['video1'][0].keys():
        for index, (point1, point2) in enumerate(zip(params['points']['video1'][0][current_frame], params['points']['video2'][0][current_frame])):
            cv2.circle(frame, point1, 2, colors[index], -1)
            cv2.circle(frame, (point2[0] + frame1.shape[1], point2[1]), 2, colors[index], -1)
            # cv2.line(frame, point1, (point2[0] + frame1.shape[1], point2[1]), colors[index], 1) 

    if previous_frame in params['points']['video1'][0].keys():
        for index, (point1, point2) in enumerate(zip(params['points']['video1'][0][previous_frame], params['points']['video2'][0][previous_frame])):
            cv2.circle(frame, point1, 2, colors[index], -1)
            cv2.circle(frame, (point2[0] + frame1.shape[1], point2[1]), 2, colors[index], -1)
            # cv2.line(frame, point1, (point2[0] + frame1.shape[1], point2[1]), colors[index], 1)

    cv2.imshow('Side-by-Side Videos', frame)

def save(dict, path):
    with open(path, 'w') as json_file:
        json.dump(dict, json_file)

def main():
    global current_frames
    global current_ret
    global selected_points
    global current_frame
    global previous_frame
    scale = 1/3
    cap1 = cv2.VideoCapture('./Axis-Wildfires/Axis-MosquitoFire/Axis-HotchkissHill.mp4')  # Path to your first video
    cap2 = cv2.VideoCapture('./Axis-Wildfires/Axis-MosquitoFire/Axis-PilotHillEldorado2.mp4')  # Path to your second video
    click_amount = 1  # Number of frames to skip when pressing 'n'
    points = {'video1': [{}], 'video2': [{}]}  # Dictionary to store clicked points
    starting_frame = 0

    camera1 = [39.009530, -120.980912,-92.32 * (math.pi/180), -6 * (math.pi/180), 2.308/111]
    camera2 = [38.820747, -121.029916, 48.58 * (math.pi/180), 5.17 * (math.pi/180), 2.298/111]

    while True:
        if current_frame < starting_frame:
            ret1, frame1 = cap1.read()
            ret2, frame2 = cap2.read()
            frame1 = cv2.resize(frame1, None, fx=scale, fy=scale, interpolation=cv2.INTER_AREA)
            frame2 = cv2.resize(frame2, None, fx=scale, fy=scale, interpolation=cv2.INTER_AREA)

            current_frame += 1
        elif current_frame == starting_frame:
            cap1_width = int(cap1.get(cv2.CAP_PROP_FRAME_WIDTH) * scale)
            cv2.namedWindow('Side-by-Side Videos')
            cv2.setMouseCallback('Side-by-Side Videos', mouse_callback, {'points': points, 'cf': current_frames, "cr":current_ret, 'cap1_width': cap1_width})
            current_frame += 1
        else:
            key = cv2.waitKey(0) & 0xFF
            redraw({'points': points, 'cf': current_frames, "cr":current_ret})

            if key == ord('n'):
                if current_frame in points["video1"][0].keys():
                        # external script that renders a point cloud and saves it based on selection
                        fullCloud(points["video1"][0][current_frame], points["video2"][0][current_frame], frame1.shape, frame2.shape, camera1, camera2, 62.5, 62.5/2, f"./Axis-Wildfires/Axis-MosquitoFire/DepthRenderings/{current_frame}")

                # Progressing to the next frames+n frame
                previous_frame = current_frame
                for _ in range(click_amount):
                    ret1, frame1 = cap1.read()
                    ret2, frame2 = cap2.read()
                    frame1 = cv2.resize(frame1, None, fx=scale, fy=scale, interpolation=cv2.INTER_AREA)
                    frame2 = cv2.resize(frame2, None, fx=scale, fy=scale, interpolation=cv2.INTER_AREA)
                    current_frame += 1
                    redraw({'points': points, 'cf': current_frames, "cr":current_ret})
                current_frames = [frame1, frame2]
                current_ret = [ret1, ret2]
            elif key == ord('q'):
                break
            elif key==32:
                # Confirms a selection
                if selected_points[0] != None and selected_points[1] != None:
                    if current_frame not in points["video1"][0].keys():
                        points["video1"][0].update({current_frame:[selected_points[0]]})
                        points["video2"][0].update({current_frame:[selected_points[1]]})
                    else:
                        points["video1"][0][current_frame].append(selected_points[0])
                        points["video2"][0][current_frame].append(selected_points[1])
                    selected_points = [None, None]
                    redraw({'points': points, 'cf': current_frames, "cr":current_ret})
            elif key == ord('z'):
                save(points, "./cesium/dataPoints.json")
                break
    cap1.release()
    cap2.release()
    cv2.destroyAllWindows()

if __name__ == "__main__":
    main()
